#version 440

layout(rgba8, binding = 4) uniform image2D img;
layout(rgba8, binding = 5) uniform image2D imgdst;
layout(local_size_x = 20, local_size_y = 20, local_size_z = 1) in;

// https://github.com/hughsk/glsl-hsv2rgb
// Used for changing the light colour, from: https://www.shadertoy.com/view/3tG3DD
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}
float luminance(vec3 col)
{
    return dot(col, vec3(0.299, 0.587, 0.114));
}
// Use pixel luminance for height
float texelHeight(vec4 rgba)
{
    return luminance(rgba.rgb);
}
uniform int NX;
uniform int NY;
#define W NX
#define H NY

void main()
{
    int i, j;
    i = int(gl_GlobalInvocationID.x);
    j = int(gl_GlobalInvocationID.y);
    vec2 pos = vec2(i, j);
    //const int W = 1024;         // ugly
   // const int H = 1024;
    vec2 res = vec2(W, H);
    //vec4 col = imageLoad(img, ivec2(gl_GlobalInvocationID.xy));
    // bump mapping 
    //https://www.shadertoy.com/view/3tG3DD



    const float normal_strength = 4.0;
    const float height_multiplier = 1.0;
    const float texel_offset = 0.5;
    vec2 uv = pos / res;
    vec3 offset = vec3(texel_offset / res, 0.0);
    vec3 surfaceColor = imageLoad(img, ivec2(pos)).rgb;
    float height = texelHeight(surfaceColor.rgbb) * height_multiplier;
    vec3 worldPosition = vec3(uv, height);
    // normal
    float ddX = texelHeight(imageLoad(img, ivec2(pos + offset.xz))) - texelHeight(imageLoad(img, ivec2(pos - offset.xz)));
    float ddY = texelHeight(imageLoad(img, ivec2(pos + offset.zy))) - texelHeight(imageLoad(img, ivec2(pos - offset.zy)));
    ddX *= normal_strength / texel_offset;
    ddY *= normal_strength / texel_offset;
    vec3 tangent = normalize(vec3(1.0, 0.0, ddX));
    vec3 bitangent = normalize(vec3(0.0, 1.0, ddY));
    vec3 normal = cross(tangent, bitangent);

    vec2 lightPosition = vec2(0.5, 0.5);//cos(iTime), sin(iTime));
    //vec3 lightColor = hsv2rgb(vec3(mod(10.0 * 0.1, 1.0), 0.1, 1.0));//hsv2rgb(vec3(mod(iTime * 0.1, 1.0), 0.1, 1.0));
    vec3 lightColor = hsv2rgb(vec3(0.1, 0.1, 1.0));//hsv2rgb(vec3(mod(iTime * 0.1, 1.0), 0.1, 1.0));
    float lightStrength = 10.0;
    float lightHeight = 2.8;
    vec3 lightDir = normalize(vec3(lightPosition, lightHeight) - worldPosition);
    float attenuation = lightStrength / pow(distance(lightPosition, uv) + lightHeight, 2.0);
    float lambert = max(0.0, dot(lightDir, normal));
    float blinnPhong = pow(max(dot(normalize(vec3(vec3(0.0, 0.0, 1.0)) + normal), lightDir), 0.0), 10.0);
    float ambient = 0.03;
    float diffuse = 0.35 * lambert;
    float specular = 0.65 * blinnPhong;
    vec3 col = ambient + (diffuse * surfaceColor + specular) * attenuation * lightColor;

    float luma = luminance(col);
    luma *= 27.5;
    col /= luma / (luma + 1.0);

    //col = surfaceColor;

    //vec2 c = vec2(W/2,H/2);
    //float d = length(c-pos)/W;
    //col = (1.0-0.4*d)*col;
    //col = (float(i)/W)*col;
    //vec4 col = color(1.51*a+1.062*b);//b*0.8+a*1.3);

    imageStore(imgdst, ivec2(gl_GlobalInvocationID.xy), vec4(col.rgb, 1.0));
}

